/* spfestival.c
 *
 * Copyright 2001, 2002 Sun Microsystems, Inc.,
 * Copyright 2001, 2002 BAUM Retec, A.G.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "spfestival.h"


/* #define FVL_DEBUG */

/* __ GLOBALS ___*/

int 		FestivalSock = -1;
/* FILE*	FestivalStream = NULL;*/


/* __ API ___*/

void fvl_write (const char *string)
{
	if ( (FestivalSock != -1) && string)
	{
#ifdef FVL_DEBUG
		fprintf(stdout, "FVL_WRITE:%s\n", string);
#endif		
		/* send (FestivalSock, string, strlen(string), 0);	 !!! TBR !!! flags		*/
		write (FestivalSock, string, strlen(string));		
		/* fprintf (FestivalStream, "%s\n", string);
		 fflush (FestivalStream);*/
	}
}

void fvl_say (SRS_VOICE *voice, const char *string)
{

	int tix;
	int i, n;
	char * out_txt;
	static char old_voice[32];
	static char voice_cmd[64];
	static char ratebuff[64]; /* nasty hack, remove this */
	static guint old_rate = 0;
	

#ifdef FVL_DEBUG
	fprintf (stderr, "fvl_say: voice:\"%s\" text:\"%s\"\n",
		 voice->TTSVoiceName, string);	
#endif
	
	/* select voice */
	if (voice->TTSVoiceName &&																					/* voice specified ? */
			(strncmp (voice->TTSVoiceName, old_voice,
				  strlen(voice->TTSVoiceName)) != 0)		/* voice changed ? */
	   )
	{		
		/* select new voice */
	
		snprintf (voice_cmd, 64, "(voice.select '%s)", voice->TTSVoiceName);
		fvl_write (voice_cmd);
		strncpy (old_voice, voice->TTSVoiceName, 64);
	}
	if (voice->Rate != old_rate)
	{
		snprintf(ratebuff, 64, "(Parameter.set 'Duration_Stretch %.2f)",
			 voice->Rate/100.0);		
		fvl_write ("(Parameter.set 'Duration_Model 'Tree_ZScore)");
		fvl_write (ratebuff);
		old_rate = voice->Rate;
	}

	/* output text */
	if (string)
	{
		/* alloc memory for output string */
		n = strlen(string);
		out_txt = (char*) malloc ((2 * n + 64) * sizeof(char));
		tix = 0;
    	        /* sprintf (&out_txt[tix], "(SayText \"%s\")", string); */
    	
		/* prepare the output string (!!! TBR !!! how ???) */
		tix += sprintf (&out_txt[tix], "(SayText \"");	/* prolog*/
		
		/* backslash and quote doubling  (!!! TBR !!! ??? )*/
		for (i = 0; i < n; ++i)
		{
	    		if (string[i] == '\\' || string[i] == '\"')
			{
				out_txt[tix++] = ' '; /* !!! TBR !!! */
			}
			else
			{
				out_txt[tix++] = string[i];
			}
			/*
			out_txt[tix++] = string[i];
			if ( string[i] == '\\' || string[i] == '"' )
			{
				out_txt[tix++] = string[i];
			}
			*/
		}
		tix += sprintf (&out_txt[tix], "\")");/* epilog*/
    	
		/* output prepared string to festival*/
		fvl_write (out_txt);
		
		/* free */
		free(out_txt);
	}

}

void fvl_speak (SRS_VOICE* voice, SRS_TEXT* text )
{
	/* !!! TBI !!! map all voice parameter to the acutal voice here*/
	if (text)
	{
		if (voice)
		{
			fvl_say (voice, text->Text);	/* !!! TEMP ONLY, need something better !!! */
		}
		else
		{
			fvl_say (voice, text->Text);	/* !!! TEMP ONLY, need something better !!! */
		}
	}
}

void fvl_shut_up()
{
  fvl_write ("(audio_mode 'shutup)");
}


int fvl_init (TTS_ENGINE* engine)
{
	
	/* int i;*/
	struct sockaddr_in serv_addr;				

#ifdef FVL_DEBUG	
	fprintf(stderr,"Welcome to festival\n");	
#endif	
	/* fvl_info->server_host = FESTIVAL_DEFAULT_SERVER_HOST;
	   fvl_info->server_port = FESTIVAL_DEFAULT_SERVER_PORT;
	   fvl_info->text_mode = FESTIVAL_DEFAULT_TEXT_MODE;
	
	   fvl->server_fd = -1;
	
	   FestivalSock = socket (PF_INET, SOCK_STREAM, 0);
	*/
	FestivalSock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
	
	if (FestivalSock >= 0)
	{
		
		memset (&serv_addr, 0, sizeof(serv_addr));		
		
		/* serverhost = gethostbyname("localhost");
		   if (serverhost != (struct hostent*)0)
		   {
		 	memmove (&serv_addr.sin_addr, serverhost->h_addr, serverhost->h_length);
		*/		
			serv_addr.sin_family = AF_INET;
			serv_addr.sin_port = htons (1314); 	/* name.sin_port = htons (FESTIVAL_DEFAULT_PORT); */
			
			serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);
			
		
   			if (connect ( FestivalSock,
   				      (struct sockaddr *) &serv_addr,
   				      sizeof (serv_addr)
   				     ) == 0
   		           )
   			{
#ifdef FVL_DEBUG   				
				fprintf(stdout, "Connected to festival\n");
#endif   			   				
   				/* FestivalStream = fdopen (dup(FestivalSock), "wb");  */
   				fvl_write ("(audio_mode 'async)");
   			
   				/* fvl_write ("(Parameter.set 'Duration_Model 'Tree_ZScore)");
   				   fvl_write ("(Parameter.set 'Duration_Stretch 0.75)");
   			
   				   fvl_write ("(intro)");
   				*/

   				/* !!! TBI !!! store the callback address */
   				
   				/* fill the engine structure with the functions */
   				engine->ShutUp    = fvl_shut_up;
				engine->Speak     = fvl_speak;
				engine->Terminate = fvl_terminate;
   			   			
   			}
   			else
   			{
   				fprintf(stderr, "Could not connect to festival\n");		
   			}
			
		/* }
		   else
		   {
			    fprintf(stderr, "Can not found local host\n");		
		   }
		*/
				
	}
	else
	{
		fprintf(stderr, "Cannot create socket\n");	
	}

	return (FestivalSock != -1);

}

void fvl_terminate ()
{
	/* !!! TBI !!! - more */
	/* fclose (FestivalStream); */
}

